package cn.newfeifan.mall.module.distri.service.applicationforwithdrawal;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.applicationforwithdrawal.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.applicationforwithdrawal.ApplicationForWithdrawalDO;
import cn.newfeifan.mall.module.distri.dal.mysql.applicationforwithdrawal.ApplicationForWithdrawalMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import org.springframework.context.annotation.Import;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static org.junit.jupiter.api.Assertions.*;

/**
 * {@link ApplicationForWithdrawalServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(ApplicationForWithdrawalServiceImpl.class)
public class ApplicationForWithdrawalServiceImplTest extends BaseDbUnitTest {

    @Resource
    private ApplicationForWithdrawalServiceImpl applicationForWithdrawalService;

    @Resource
    private ApplicationForWithdrawalMapper applicationForWithdrawalMapper;

    @Test
    public void testCreateApplicationForWithdrawal_success() {
        // 准备参数
        ApplicationForWithdrawalSaveReqVO createReqVO = randomPojo(ApplicationForWithdrawalSaveReqVO.class).setId(null);

        // 调用
        Long applicationForWithdrawalId = applicationForWithdrawalService.createApplicationForWithdrawal(createReqVO);
        // 断言
        assertNotNull(applicationForWithdrawalId);
        // 校验记录的属性是否正确
        ApplicationForWithdrawalDO applicationForWithdrawal = applicationForWithdrawalMapper.selectById(applicationForWithdrawalId);
        assertPojoEquals(createReqVO, applicationForWithdrawal, "id");
    }

    @Test
    public void testUpdateApplicationForWithdrawal_success() {
        // mock 数据
        ApplicationForWithdrawalDO dbApplicationForWithdrawal = randomPojo(ApplicationForWithdrawalDO.class);
        applicationForWithdrawalMapper.insert(dbApplicationForWithdrawal);// @Sql: 先插入出一条存在的数据
        // 准备参数
        ApplicationForWithdrawalSaveReqVO updateReqVO = randomPojo(ApplicationForWithdrawalSaveReqVO.class, o -> {
            o.setId(dbApplicationForWithdrawal.getId()); // 设置更新的 ID
        });

        // 调用
        applicationForWithdrawalService.updateApplicationForWithdrawal(updateReqVO);
        // 校验是否更新正确
        ApplicationForWithdrawalDO applicationForWithdrawal = applicationForWithdrawalMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, applicationForWithdrawal);
    }

    @Test
    public void testUpdateApplicationForWithdrawal_notExists() {
        // 准备参数
        ApplicationForWithdrawalSaveReqVO updateReqVO = randomPojo(ApplicationForWithdrawalSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> applicationForWithdrawalService.updateApplicationForWithdrawal(updateReqVO), APPLICATION_FOR_WITHDRAWAL_NOT_EXISTS);
    }

    @Test
    public void testDeleteApplicationForWithdrawal_success() {
        // mock 数据
        ApplicationForWithdrawalDO dbApplicationForWithdrawal = randomPojo(ApplicationForWithdrawalDO.class);
        applicationForWithdrawalMapper.insert(dbApplicationForWithdrawal);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbApplicationForWithdrawal.getId();

        // 调用
        applicationForWithdrawalService.deleteApplicationForWithdrawal(id);
       // 校验数据不存在了
       assertNull(applicationForWithdrawalMapper.selectById(id));
    }

    @Test
    public void testDeleteApplicationForWithdrawal_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> applicationForWithdrawalService.deleteApplicationForWithdrawal(id), APPLICATION_FOR_WITHDRAWAL_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetApplicationForWithdrawalPage() {
       // mock 数据
       ApplicationForWithdrawalDO dbApplicationForWithdrawal = randomPojo(ApplicationForWithdrawalDO.class, o -> { // 等会查询到
           o.setUserId(null);
           o.setWithdrawalType(null);
           o.setAmount(null);
           o.setWithdrawalAccount(null);
           o.setStatus(null);
           o.setApplicationForWithdrawalChannelId(null);
           o.setCreateTime(null);
       });
       applicationForWithdrawalMapper.insert(dbApplicationForWithdrawal);
       // 测试 userId 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setUserId(null)));
       // 测试 withdrawalType 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setWithdrawalType(null)));
       // 测试 amount 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setAmount(null)));
       // 测试 withdrawalAccount 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setWithdrawalAccount(null)));
       // 测试 status 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setStatus(null)));
       // 测试 applicationForWithdrawalChannelId 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setApplicationForWithdrawalChannelId(null)));
       // 测试 createTime 不匹配
       applicationForWithdrawalMapper.insert(cloneIgnoreId(dbApplicationForWithdrawal, o -> o.setCreateTime(null)));
       // 准备参数
       ApplicationForWithdrawalPageReqVO reqVO = new ApplicationForWithdrawalPageReqVO();
       reqVO.setUserId(null);
       reqVO.setWithdrawalType(null);
       reqVO.setAmount(null);
       reqVO.setWithdrawalAccount(null);
       reqVO.setStatus(null);
       reqVO.setApplicationForWithdrawalChannelId(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<ApplicationForWithdrawalDO> pageResult = applicationForWithdrawalService.getApplicationForWithdrawalPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbApplicationForWithdrawal, pageResult.getList().get(0));
    }

}