package cn.newfeifan.mall.module.distri.service.ptdailywithdrawal;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.ptdailywithdrawal.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.ptdailywithdrawal.PtDailyWithdrawalDO;
import cn.newfeifan.mall.module.distri.dal.mysql.ptdailywithdrawal.PtDailyWithdrawalMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import org.springframework.context.annotation.Import;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static org.junit.jupiter.api.Assertions.*;

/**
 * {@link PtDailyWithdrawalServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(PtDailyWithdrawalServiceImpl.class)
public class PtDailyWithdrawalServiceImplTest extends BaseDbUnitTest {

    @Resource
    private PtDailyWithdrawalServiceImpl ptDailyWithdrawalService;

    @Resource
    private PtDailyWithdrawalMapper ptDailyWithdrawalMapper;

    @Test
    public void testCreatePtDailyWithdrawal_success() {
        // 准备参数
        PtDailyWithdrawalSaveReqVO createReqVO = randomPojo(PtDailyWithdrawalSaveReqVO.class).setId(null);

        // 调用
        Long ptDailyWithdrawalId = ptDailyWithdrawalService.createPtDailyWithdrawal(createReqVO);
        // 断言
        assertNotNull(ptDailyWithdrawalId);
        // 校验记录的属性是否正确
        PtDailyWithdrawalDO ptDailyWithdrawal = ptDailyWithdrawalMapper.selectById(ptDailyWithdrawalId);
        assertPojoEquals(createReqVO, ptDailyWithdrawal, "id");
    }

    @Test
    public void testUpdatePtDailyWithdrawal_success() {
        // mock 数据
        PtDailyWithdrawalDO dbPtDailyWithdrawal = randomPojo(PtDailyWithdrawalDO.class);
        ptDailyWithdrawalMapper.insert(dbPtDailyWithdrawal);// @Sql: 先插入出一条存在的数据
        // 准备参数
        PtDailyWithdrawalSaveReqVO updateReqVO = randomPojo(PtDailyWithdrawalSaveReqVO.class, o -> {
            o.setId(dbPtDailyWithdrawal.getId()); // 设置更新的 ID
        });

        // 调用
        ptDailyWithdrawalService.updatePtDailyWithdrawal(updateReqVO);
        // 校验是否更新正确
        PtDailyWithdrawalDO ptDailyWithdrawal = ptDailyWithdrawalMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, ptDailyWithdrawal);
    }

    @Test
    public void testUpdatePtDailyWithdrawal_notExists() {
        // 准备参数
        PtDailyWithdrawalSaveReqVO updateReqVO = randomPojo(PtDailyWithdrawalSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> ptDailyWithdrawalService.updatePtDailyWithdrawal(updateReqVO), PT_DAILY_WITHDRAWAL_NOT_EXISTS);
    }

    @Test
    public void testDeletePtDailyWithdrawal_success() {
        // mock 数据
        PtDailyWithdrawalDO dbPtDailyWithdrawal = randomPojo(PtDailyWithdrawalDO.class);
        ptDailyWithdrawalMapper.insert(dbPtDailyWithdrawal);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbPtDailyWithdrawal.getId();

        // 调用
        ptDailyWithdrawalService.deletePtDailyWithdrawal(id);
       // 校验数据不存在了
       assertNull(ptDailyWithdrawalMapper.selectById(id));
    }

    @Test
    public void testDeletePtDailyWithdrawal_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> ptDailyWithdrawalService.deletePtDailyWithdrawal(id), PT_DAILY_WITHDRAWAL_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetPtDailyWithdrawalPage() {
       // mock 数据
       PtDailyWithdrawalDO dbPtDailyWithdrawal = randomPojo(PtDailyWithdrawalDO.class, o -> { // 等会查询到
           o.setAmount(null);
           o.setCount(null);
           o.setStatus(null);
           o.setStartTime(null);
           o.setEndTime(null);
           o.setCreateTime(null);
       });
       ptDailyWithdrawalMapper.insert(dbPtDailyWithdrawal);
       // 测试 amount 不匹配
       ptDailyWithdrawalMapper.insert(cloneIgnoreId(dbPtDailyWithdrawal, o -> o.setAmount(null)));
       // 测试 count 不匹配
       ptDailyWithdrawalMapper.insert(cloneIgnoreId(dbPtDailyWithdrawal, o -> o.setCount(null)));
       // 测试 status 不匹配
       ptDailyWithdrawalMapper.insert(cloneIgnoreId(dbPtDailyWithdrawal, o -> o.setStatus(null)));
       // 测试 startTime 不匹配
       ptDailyWithdrawalMapper.insert(cloneIgnoreId(dbPtDailyWithdrawal, o -> o.setStartTime(null)));
       // 测试 endTime 不匹配
       ptDailyWithdrawalMapper.insert(cloneIgnoreId(dbPtDailyWithdrawal, o -> o.setEndTime(null)));
       // 测试 createTime 不匹配
       ptDailyWithdrawalMapper.insert(cloneIgnoreId(dbPtDailyWithdrawal, o -> o.setCreateTime(null)));
       // 准备参数
       PtDailyWithdrawalPageReqVO reqVO = new PtDailyWithdrawalPageReqVO();
       reqVO.setAmount(null);
       reqVO.setCount(null);
       reqVO.setStatus(null);
       reqVO.setStartTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setEndTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<PtDailyWithdrawalDO> pageResult = ptDailyWithdrawalService.getPtDailyWithdrawalPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbPtDailyWithdrawal, pageResult.getList().get(0));
    }

}